// Copyright 2018-2023 the Deno authors. All rights reserved. MIT license.
//!  This example shows that op-panics currently result in UB (likely "failed to initiate panic")
//!  without a custom panic hook that aborts the process or -C panic=abort.
//!
//!  This happens due to the UB of panicking in an extern "C",
//!  given how ops are reduced via rusty_v8::MapFnTo
//!  See:
//!    - https://github.com/rust-lang/rust/issues/74990
//!    - https://rust-lang.github.io/rfcs/2945-c-unwind-abi.html

use deno_core::*;

fn main() {
  #[op2(fast)]
  fn op_panik() {
    panic!("panik !!!")
  }

  let extensions = vec![Extension {
    name: "my_ext",
    ops: std::borrow::Cow::Borrowed(&[op_panik::DECL]),
    ..Default::default()
  }];
  let mut rt = JsRuntime::new(RuntimeOptions {
    extensions,
    ..Default::default()
  });
  rt.execute_script_static("panik", "Deno.core.ops.op_panik()")
    .unwrap();
}
